/*
 * XMLPath.java
 *
 * Created on February 17, 2002, 7:57 AM
 */

import java.util.*;
import org.xml.sax.*;

/** Class to generate and store all paths pertaining to the XML file being read in by the parser
 *
 * @author Chris Schmidt
 */
public class XMLPath {
    
    private Vector pathList;    
    private Stack pathStack;
    
    /** Creates a new instance of XMLPath */
    public XMLPath() {
        pathList = new Vector();
        pathStack = new Stack();
    }
    
    /** Push an element onto the internal stack
     * @param element Element name of new element to push onto the stack
     */    
    public void pushElement(String element) {
        // Push a new element name onto the stack
        pathStack.push(element);
    }
    
    /** Remove the named element from the stack
     * @param element Element to be removed.  If element names are different, then the XML file is not well-formed
     */    
    public void popElement(String element) {
        // Pop an element off the stack, but ensure that the 
        //  element that needs to come off matches the element
        //  name passed in (just to be sure)
        String topElement = (String)pathStack.peek();
        if (topElement.equals(element)) {
            pathStack.pop();
        } else {
            System.err.println("Error while removing elements. Expected " +
                "element " + element + ", but found element " + topElement);
            System.err.println("Unable to pop element from stack");
        }
    }
    
    /** Add an element into the path list vector
     * @param element The element's name
     * @param attList Attribute list for the named element
     */    
    public void addElement(String element, Attributes attList) {
        // Local Variables
        String path;
        String tempPath;
        String tempAtt;
        boolean bAlreadyAdded = false;
        
        // Build a string representing the path of the element
        path = getPath(null);
        
        // Check the vector to see if this path has already been added
        for (Enumeration e = pathList.elements(); e.hasMoreElements();) {
            tempPath = (String)e.nextElement();
            
            if (tempPath.equals(path)) {
                bAlreadyAdded = true;
            }
        }
        
        // If we haven't added this element already, do so now
        if (bAlreadyAdded == false) {
            pathList.add(path);
        
            // Also add any attributes the element may have
            for (int loop = 0; loop < attList.getLength(); loop++) {
                tempAtt = attList.getQName(loop);
                pathList.add(path + "#/@" + tempAtt);
            }
        }
    }
    
    /** Returns an enumeration of all paths collected
     * @return Enumeration Returns all paths collected thus far
     */    
    public Enumeration getPathList() {
        // Return all the paths that have been found in Enumeration format
        return pathList.elements();
    }
    
    /** Returns the path of the current element on the stack, or the attribute if the attName variable is not null
     * @param attName Name of an attribute of the current element on the stack.  If null, return the path to just the element on the stack
     * @return String Fully formed path of the element currently on the stack
     */    
    public String getPath(String attName) {
        String path = "";
        // Returns a pathname for the current element (or element's attribute)
        // if attName isn't a null, then the user is requesting a path to
        // an attribute.  Otherwise, the request is for the current element
        // path (as per the internal stack variable)
        
        // Note: this method assumes that the method pushElement has already been
        // called for the element in question.
        for (Enumeration e = pathStack.elements();e.hasMoreElements();) {
            path = path + "#/" + (String)e.nextElement();
        }
        // If the user wants the path for an attribute, add the att name
        if (attName != null) {
            path = path + "#/@" + attName;
        }
        
        return path;
    }
    
    /** Gets the path index (starting at 1..n) of the current element on the stack.  Or the attribute underneat the element if the attName variable is not null.
     * @param attName Name of an attribute that is a child of the current element on the stack.  Can be null to denote get the index of the element itself
     * @return int
     */    
    public int getPathIndex(String attName) {
        // Returns a 1-base index of the current path (or attribute of the
        // current path, if attName is not null).
        String path = getPath(attName);
        String tempPath;
        int index = 1;
        int result = -1;
        
        // We have the current path, go through the vector to find the 
        // path that matches
        for (Enumeration e = pathList.elements(); e.hasMoreElements();) {
            tempPath = (String)e.nextElement();
            if (tempPath.equals(path)) {
                result = index;
            }
            index++;
        }
        // If the path was not found, an error occurred, and the result will
        // be a negative 1.
        return result;
    }
}
